// $Id$

// Copyright (C) 2009 Dr. Georg Troska <georg.troska@tu-dortmund.de>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include "adc.h"


void ADC_init(void) {
	uint16_t result;

	ADMUX = (0<<REFS1) | (1<<REFS0);      // AVcc als Referenz benutzen
	// ADMUX = (1<<REFS1) | (1<<REFS0);      // interne Referenzspannung nutzen
	ADCSRA = (1<<ADPS1) | (1<<ADPS0);     // Frequenzvorteiler
	ADCSRA |= (1<<ADEN);                  // ADC aktivieren

	/* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
	also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */

	ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung 
	while (ADCSRA & (1<<ADSC) ) {}        // auf Abschluss der Konvertierung warten
	/* ADCW muss einmal gelesen werden, sonst wird Ergebnis der nchsten
	Wandlung nicht bernommen. */
	result = ADCW;
}
 
uint16_t ADC_read(uint8_t channel) {
	// Kanal waehlen, ohne andere Bits zu beeinflu§en
	ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
	ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
	while (ADCSRA & (1<<ADSC) ) {}  // auf Abschluss der Konvertierung warten
	return ADCW;                    // ADC auslesen und zurckgeben
}
 
uint16_t ADC_readAvg(uint8_t channel, uint8_t average) {
	uint32_t result = 0;
	uint8_t i;
	for ( i = 0; i < average; ++i )
		result += ADC_read( channel );

	return (uint16_t)( result / average );
}
