// $Id$

// Copyright (C) 2009 Dr. Georg Troska <georg.troska@tu-dortmund.de>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include "i2c.h"
#include <util/delay.h>


#define SDA   3
#define SCL   4
 
#define READ_PCF    0xa1		
#define WRITE_PCF   0xa0		
 
unsigned char ack=0;
unsigned char data[5]={};
unsigned char myRegister;	

const uint8_t I2CinvFreq = 10;		


		

void I2CstartCondition (void) {
	DDRD |= (1 << SDA) | (1 << SCL);
	PORTD |= (1 << SDA) | (1 << SCL);
	_delay_us(I2CinvFreq);
	PORTD &= ~(1 << SDA);
	_delay_us(I2CinvFreq);
	PORTD &= ~(1 << SCL);
}
 
void I2CstopCondition (void) {
	PORTD |= (1 << SCL);
	_delay_us(I2CinvFreq);
	PORTD &= ~(1 << SDA);
	_delay_us(I2CinvFreq);
	PORTD |= (1 << SDA);
}
 
void I2CsendByte (unsigned char data) {
	unsigned char i;
	for (i=0;i<8;i++) {
		if ((data & 0x80) == 0x00) {
			PORTD &=~ (1 << SDA);
		} else {
			PORTD |= (1 << SDA);
		}
		_delay_us(I2CinvFreq);
		PORTD |= (1 << SCL);
		_delay_us(I2CinvFreq);
		PORTD &= ~(1 << SCL);
		data = data << 1;
	}
	DDRD &= ~(1 << SDA);
	PORTD |= (1 << SDA);

	_delay_us(I2CinvFreq);
	PORTD |= (1 << SCL);
	_delay_us(I2CinvFreq);
	if (( PIND & (1 << SDA) ) == (1 << SDA)) {
		ack=1;
	}
	PORTD &= ~(1 << SCL);

	PORTD &= ~(1 << SDA);
	DDRD |= (1 << SDA);
}
 
unsigned char I2CgetByte (void) { 
	unsigned char i, res=0;
 
	DDRD &= ~(1 << SDA);
	PORTD |= (1 << SDA);

	for (i = 0; i < 8; i++) {
		res = res << 1;
		PORTD |= (1 << SCL);
		_delay_us(I2CinvFreq);
		if ( (PIND & (1 << SDA) ) == (1 << SDA) ) {
			res = res | 0x01;
		}
		PORTD &= ~(1 << SCL);
		_delay_us(I2CinvFreq);
	}

	PORTD &= ~(1 << SDA);	
	DDRD |= (1 << SDA); 

	_delay_us(I2CinvFreq);
	PORTD |= (1 << SCL);
	_delay_us(I2CinvFreq);
	PORTD &= ~(1 << SCL);

	return res;
}
 
char * I2CreadDateTime (void) { 
	unsigned char i;
 
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x02);

	I2CstartCondition();
	I2CsendByte(READ_PCF);
	for (i = 0; i < 5; i++)
		data[i] = I2CgetByte();
	I2CstopCondition();
	return &data;
}
 
void I2CwriteDateTime (char myData[]) { 
	unsigned char i;
	
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x02);

	for (i = 0; i < 5; i++)
		I2CsendByte (myData[i]);
	I2CstopCondition();
}

void I2CgetTime(uint8_t * hour, uint8_t * min, uint8_t * sec) {
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x02);

	I2CstartCondition();
	I2CsendByte(READ_PCF);
	
	myRegister = I2CgetByte(); // Register 0x02
	(*sec) = (myRegister & 0x0f) + 10*((myRegister >> 4) & 0x07);
	
	myRegister = I2CgetByte(); // Register 0x03
	(*min) = (myRegister & 0x0f) + 10*((myRegister >> 4) & 0x07);
	
	myRegister = I2CgetByte(); // Register 0x04
	(*hour) = (myRegister & 0x0f) + 10*((myRegister >> 4) & 0x03);
		
	I2CstopCondition();
}

void I2CgetDate(uint8_t * year, uint8_t * month, uint8_t * day) {
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x05);

	I2CstartCondition();
	I2CsendByte(READ_PCF);
	
	myRegister = I2CgetByte(); // Register 0x05
	(*day) = (myRegister & 0x0f) + 10*((myRegister>> 4) & 0x03);
	(*year) = ((myRegister >> 6) & 0x03);
	
	myRegister = I2CgetByte(); // Register 0x06
	(*month) = (myRegister & 0x0f) + 10*((myRegister >> 4) & 0x01);
	
	I2CstopCondition();
	/* Read the year from the NVRAM */
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x10); // The NVRAM Adress of the year

	I2CstartCondition();
	I2CsendByte(READ_PCF);
	myRegister = I2CgetByte(); // Register 0x10
	while ((*year) != (myRegister & 0x03)) {
		myRegister++;
		if (myRegister > 99) myRegister = 0;
	}
	if (myRegister > 99) myRegister = 0;
	(*year) = myRegister;
	I2CstopCondition();
	
	/* Update of the year in the NVRAM */
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x10);
	I2CsendByte (myRegister); //into Register 0x10
	I2CstopCondition();
}

void I2CgetTimeStr(char * out) {
	
	uint8_t hour;
	uint8_t min;
	uint8_t sec;
	
	I2CgetTime(&hour,&min,&sec);
	
	sprintf(out,"%02d:%02d:%02d",hour,min,sec);	
}

void I2CgetDateStr(char * out) {
	
	uint8_t year;
	uint8_t month;
	uint8_t day;
	
	I2CgetDate(&year, &month, &day);
	
	sprintf(out,"%02d-%02d-%02d",year,month,day);	
}


void I2CsetTime(uint8_t hour, uint8_t min, uint8_t sec) {
	unsigned char i;
	
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x02);
	
	myRegister = (sec % 10) | ((sec/10)<<4);
	I2CsendByte (myRegister); //into Register 0x02
		
	myRegister = (min % 10) | ((min/10)<<4);
	I2CsendByte (myRegister); //into Register 0x03
	
	myRegister = (hour % 10) | ((hour/10)<<4);
	I2CsendByte (myRegister); //into Register 0x04
		
		
	I2CstopCondition();
}

void I2CsetDate(uint8_t year, uint8_t month, uint8_t day) {
	unsigned char i;
	
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x05);
	
	myRegister = (day % 10) | ((day/10)<<4) | (year << 6);
	I2CsendByte (myRegister); //into Register 0x05
		
	myRegister = (month % 10) | ((month/10)<<4);
	I2CsendByte (myRegister); //into Register 0x06
	
	I2CstopCondition();
	
	
	/* The year is stored in the NVRAM of the PCF8583 */
	I2CstartCondition();
	I2CsendByte (WRITE_PCF);
	I2CsendByte (0x10); //NVRAM Adress for the year
	
	myRegister = year;
	I2CsendByte (myRegister); //into Register 0x04
	I2CstopCondition();
		
		
	
}


