// $Id$

// Copyright (C) 2009 Dr. Georg Troska <georg.troska@tu-dortmund.de>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include "lcd.h"
#include "spi.h"

void lcd_data(unsigned char temp1){
   unsigned char temp2 = temp1;
 
   LCD_PORT |= (1<<LCD_RS);        // RS auf 1 setzen
   SPI_sendToLCD(LCD_PORT);
 
   temp1 = temp1 >> 4;
   temp1 = temp1 & 0x0F;
   LCD_PORT &= 0xF0;
   SPI_sendToLCD(LCD_PORT);

   LCD_PORT |= temp1;               // setzen
   SPI_sendToLCD(LCD_PORT);

   lcd_enable();
 
   temp2 = temp2 & 0x0F;
   LCD_PORT &= 0xF0;
   SPI_sendToLCD(LCD_PORT);
   LCD_PORT |= temp2;               // setzen
   SPI_sendToLCD(LCD_PORT);
   lcd_enable();
   
   _delay_us(46);
}
 

void lcd_command(unsigned char temp1) {
   unsigned char temp2 = temp1;
 
   LCD_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen
   SPI_sendToLCD(LCD_PORT);
 
   temp1 = temp1 >> 4;              // oberes Nibble holen
   temp1 = temp1 & 0x0F;            // maskieren
   LCD_PORT &= 0xF0;
   SPI_sendToLCD(LCD_PORT);
   LCD_PORT |= temp1;               // setzen
   SPI_sendToLCD(LCD_PORT);
   lcd_enable();
 
   temp2 = temp2 & 0x0F;            // unteres Nibble holen und maskieren
   LCD_PORT &= 0xF0;
   SPI_sendToLCD(LCD_PORT);
   LCD_PORT |= temp2;               // setzen
   SPI_sendToLCD(LCD_PORT);
   lcd_enable();
   
   _delay_us(46);
}
 
void lcd_enable(void) {
   LCD_PORT |= (1<<LCD_EN);
   SPI_sendToLCD(LCD_PORT);
   _delay_us(1);         
   LCD_PORT &= ~(1<<LCD_EN);
   SPI_sendToLCD(LCD_PORT);
}

void lcd_init(void) {
   //LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) | (1<<LCD_EN);   // Port auf Ausgang schalten
 
   _delay_ms(15);
   LCD_PORT &= 0xF0;
   SPI_sendToLCD(LCD_PORT);
   LCD_PORT |= 0x03;     
   SPI_sendToLCD(LCD_PORT);
   LCD_PORT &= ~(1<<LCD_RS);      // RS auf 0
   SPI_sendToLCD(LCD_PORT);
   lcd_enable();
 
   _delay_ms(5);
   lcd_enable();
 
   _delay_ms(1);
   lcd_enable();
   _delay_ms(1);
 
   // 4 Bit Modus aktivieren 
   LCD_PORT &= 0xF0;
   SPI_sendToLCD(LCD_PORT);
   LCD_PORT |= 0x02;
   SPI_sendToLCD(LCD_PORT);
   lcd_enable();
   _delay_ms(1);
 
   // 4Bit / 2 Zeilen / 5x7
   lcd_command(0x28);
    
   // Display ein / Cursor aus / kein Blinken
   lcd_command(0x0C); 
 
   // inkrement / kein Scrollen
   lcd_command(0x06);
 
   lcd_clear();
  
}
 
void lcd_clear(void) {
   lcd_command(CLEAR_DISPLAY);
   _delay_ms(5);
}
 
void lcd_home(void) {
   lcd_command(CURSOR_HOME);
   _delay_ms(5);
}
 
void set_cursor(uint8_t x, uint8_t y) {
  uint8_t tmp;
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    case 3: tmp=0x80+0x10+x; break;    // 3. Zeile
    case 4: tmp=0x80+0x50+x; break;    // 4. Zeile
    default: return;                   // fr den Fall einer falschen Zeile
  }
  lcd_command(tmp);
}
 
void lcd_string(char *data) {
    while(*data) {
        lcd_data(*data);
        data++;
    }
}

