// $Id$

// Copyright (C) 2009 Dr. Georg Troska <georg.troska@tu-dortmund.de>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include <avr/io.h>
#include <util/delay.h>

#include <avr/eeprom.h>



#include "spi.h"
#include "lcd.h"
#include "pt1000.h"
#include "timer.h"
#include "i2c.h"
#include "menu.h"
#include "globalvars.h"
#include "interval.h"

#include "sd.h"

#define INTERRUPT1S PD2

uint8_t myStatus;

void getSQLDatime(char * out) {//Reads in new Datime from RTC and passes back the SQL-String of it
	I2CgetDate( &datime[0], &datime[1], &datime[2]);
	I2CgetTime( &datime[3], &datime[4], &datime[5]);
	sprintf(out, "%02d-%02d-%02d %02d:%02d:%02d\t", datime[0], datime[1], datime[2], datime[3], datime[4], datime[5]);
}

void getWriteFilename(char *out, uint8_t SDInverval, char prefix[], char postfix[]) {
	strcpy(out, prefix);
		char dateString[20]; 
	switch(SDInterval) {
		case k1Sec:
		case k2Sec:
		case k5Sec:
		case k10Sec:
		case k30Sec:
			sprintf(dateString,"%02d-%02d-%02d_%02dh",datime[0], datime[1], datime[2], datime[3]);
			strcat(out, dateString);
			break;
		case k1Min:
		case k2Min:
		case k5Min:
		case k10Min:
			sprintf(dateString,"%02d-%02d-%02d",datime[0], datime[1], datime[2]);
			strcat(out, dateString);
			break;
		default:
			sprintf(dateString,"%02d-%02d",datime[0], datime[1]);
			strcat(out, dateString);
			break;
			
	}
	strcat(out, postfix);
	
}

uint32_t getSecondsRemaiming(uint8_t SDInverval, uint32_t secOfThisDay) {
	uint32_t triggerPoint;
	switch (SDInterval) {
		case k1Sec:
			triggerPoint = 1;
			break;
		case k2Sec:
			triggerPoint = 2;
			break;
		case k5Sec:
			triggerPoint = 5;
			break;
		case k10Sec:
			triggerPoint = 10;
			break;
		case k30Sec:
			triggerPoint = 30;
			break;
		case k1Min:
			triggerPoint = 60;
			break;
		case k2Min:
			triggerPoint = 2*60;
			break;
		case k5Min:
			triggerPoint = 5*60;
			break;
		case k10Min:
			triggerPoint = 10*60;
			break;
		case k30Min:
			triggerPoint = 30*60;
			break;
		case k1h:
			triggerPoint = 3600;
			break;
		case k2h:
			triggerPoint = 2*3600;
			break;
		case k6h:
			triggerPoint = 6*3600;
			break;
		case k12h:
			triggerPoint = 12*3600;
			break;
		case k1d:
			triggerPoint = 24*3600;
			break;
	}
	return ((secOfThisDay+1) % triggerPoint);
}

int mainLoop(void) {
	
	lcd_clear();
	lcd_home();
	
	uint8_t pt;
	uint8_t lastwindow = 0xff;
	for (;;) {
		if (myISRs & 0x01) { //ISR for updating the LCD (~1/sec)
			myISRs &= ~(1 << 0); //reset it ISR bit
			updateWindow(window);
		}
		if (myISRs & 0x02) {//ISR for temperature readout - normal operations are only triggered by myISRs
			myISRs &= ~(1 << 1); //reset the ISR bit
			if (window == k2ndDo1cal) { // check if cal1000 needs to be updated
				showR = 9999;
				for (pt=0; pt <8; ++pt) {
					enablePt1000(pt);
					_delay_ms(1);
					uint16_t myR = getR(pt);
					if (myR < 1450) {
						isCal1000 |= (1 << pt);
						c1000[pt] = myR;
						showR = myR;
					} 
					disablePt1000();
				}
			} else if (window == k2ndDo2cal) { // check if cal1400 needs to be updated
				showR = 9999;
				for (pt=0; pt <8; ++pt) {
					enablePt1000(pt);
					_delay_ms(1);
					uint16_t myR = getR(pt);//(int16_t)getT(getR(pt))/10;
					if (myR < 1450) {
						isCal1400 |= (1 << pt);
						c1400[pt] = myR;
						showR = myR;
					} 
					disablePt1000();
				}	
			} else if (window == kSaved) {
				_delay_ms(500);
				window = k1stNormal;				
			}
			else { //this is the default case
				for (pt=0; pt <8; ++pt) {
					float m, b;
					//if (  (isCal1000 & (1 << pt) ) && (isCal1400 & (1 << pt)) ) { //correction only when calibration for both points are available
					if (1) {
						m = (float)(1400-1000)/(c1400[pt] - c1000[pt]);
						b = 1000. - m*c1000[pt];
						
					} else {
						m = 1.0;
						b = 0.;
					}
					enablePt1000(pt);
					_delay_ms(1);
					uint32_t myR;
					myR = getR(pt)*m + b;
					disablePt1000();
					
					uint16_t sumTemps = 0;
					uint8_t j;
					
					//Check the old average temperature
					for (j=0; j < 10; j++) {
						sumTemps += temps[j][pt];
					}
					sumTemps /= 10;
					if (abs(getT(myR)-sumTemps) > 20) { // If the temperature difference is so high, it means that a pt1000 was connected or disconnected, change all value instantanously
						for (j=0; j < 10; j++) {
							temps[j][pt] = getT(myR);
						}
						displayStatus |= 0x08;
						myStatus |= 0x04; 
					} else {
						temps[currentTempPos][pt] = getT(myR);
					}
										
					//temps[pt] = myR;
					
				}
				currentTempPos++;
				if (currentTempPos > 9) {
					currentTempPos = 0;
					// all 10 averageing registers are used now::
					displayStatus |= 0x08;
					myStatus |= 0x04; 
					
				}
			}
		}
		
		menuLoop();
		
		// The myStatus &0x02 is necessarry, so that this routine in only called on the leading edge
		if ( (PIND & (1 << INTERRUPT1S) ) && !(myStatus & 0x02) ) {
			myStatus |= 0x02; //activate bit 1
			char myDateString[20];
			//Readin the datime from the RTC-chip into the datime-registers once per second
			//if (!(window >= 0x32 && window <= 0x37)) { //Make sure that the clock is not being set currently, otherwise it would be reset to the old values again and again
				getSQLDatime(myDateString); //This updates the datime-registers as well
			//}
			
			uint32_t secOfThisDay = (uint32_t)(datime[3]*3600) + (uint16_t)(datime[4]*60) + datime[5];
			uint32_t remainSec = getSecondsRemaiming(SDInterval, secOfThisDay);
			
			if (fs && (remainSec == 0) && ((myStatus >> 2) & 0x01) ) { //If there is a filesystem store the date and the temperatures to the SD
				
				char myTempString[7];
				char myCompleteString[8*8+22];
				int myPT;
				
				strcpy(myCompleteString, myDateString);
				for (myPT = 0; myPT < 8; myPT++) {
					int16_t myTemp = 0;
					uint8_t tempPos = 0;
					for (tempPos = 0; tempPos < 10; tempPos++) {
						myTemp += temps[tempPos][myPT];
					}
					sprintf(myTempString, "%d.%d\t", myTemp/10, myTemp-(myTemp/10)*10);
					if (myTemp/10 > 120) { //This channel is not connected, overwrite with an empty string
						sprintf(myTempString, " \t");
					}
					strcat(myCompleteString,  myTempString);
				}
				sprintf(myTempString, "\n"); //Newlinestring
				strcat (myCompleteString, myTempString);
				
				displayStatus |= 0x04; //This will write something in the display
				char filename[32];
				getWriteFilename(filename, SDInterval, "Data",".txt");
				SDappendFile(fs, "/", filename, myCompleteString);
			 }	
		}
		//This is the definition of the falling edge
		if ( !(PIND & (1 << INTERRUPT1S) ) && (myStatus & 0x02)) {
			myStatus &= ~(0x02); //deactivate bit 1
		}
	}
}

int main(void)
{
    SPI_masterInit();
    lcd_init();
    ADC_init();
    timerInit();
    
    sd_raw_init();
	uint8_t ret = 0;
	//struct partition_struct* partition = partition_open(sd_raw_read, sd_raw_read_interval, sd_raw_write, sd_raw_write_interval, 0);
	//fs = fat_open(partition);
	SDTryLoading();
	char mychar[32];
	char SQLDatime[22];
	getSQLDatime(SQLDatime);
	sprintf(mychar, "%s\tNeustart\n",SQLDatime);
	if (fs) ret = SDappendFile(fs, "/", "LOG.TXT", mychar);
   
   if (ret > 0) {
	   lcd_string("SD-Karte erkannt");
	   _delay_ms(1000);
   }
   
   
    lcd_command(0b01000000);
    lcd_data(0x12);
    lcd_data(0x15);
    lcd_data(0x04);
    lcd_data(0x04);
    lcd_data(0x04);
    lcd_data(0x05);
    lcd_data(0x02);
    lcd_data(0x00);

    DDRD = 0x00;
    PORTD = 0xff; //PullUp aktiv
    
    window = k1stNormal; //Main setup-screen
    myISRs = 0x00;
    myStatus = 0x00;
    
    lastWindow = 0x00;
    
    /* Pass initial values from EEPROM to these variables */
    uint8_t i;
    for (i=0; i < 8; i++) {
		c1000[i] = eeprom_read_word(&eeC1000[i]); 
		c1400[i] = eeprom_read_word(&eeC1400[i]);
	}
	
	isCal1000 = eeprom_read_byte(&eeIsCal1000);
    isCal1400 = eeprom_read_byte(&eeIsCal1400);
    
    SDInterval = eeprom_read_byte(&eeSDInterval);
    
    tim0count = 0;
    showR = 9999;
    currentTempPos = 0;

    
    initMenu();

	mainLoop();

   return 0;   /* never reached */
}
