// $Id$

// Copyright (C) 2009 Dr. Georg Troska <georg.troska@tu-dortmund.de>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include "menu.h"
#include "lcd.h"
#include "globalvars.h"
#include "timer.h"
#include "interval.h"
#include "sd.h"

//static const __flash char logTextCal1000[]  = "Kalibration für 1.0kOhm verändert: %d, %d, %d, %d, %d, %d, %d, %d";



void initMenu() {
	displayStatus = 0x00;
}


inline uint8_t debounce(volatile uint8_t *port, uint8_t pin)
{
    if ( !(*port & (1 << pin)) )
    {
        /* Pin wurde auf Masse gezogen, 100ms warten   */
        _delay_ms(50);   // Maximalwert des Parameters an _delay_ms 
        _delay_ms(50);   // beachten, vgl. Dokumentation der avr-libc
        while (!(*port & (1 << pin)) ) _delay_us(10);
        if ( *port & (1 << pin) )
        {
            /* Anwender Zeit zum Loslassen des Tasters geben */
            _delay_ms(50);
            _delay_ms(50); 
            return 1;
        }
    }
    return 0;
}

int updateWindow(uint8_t myWindow) {
	char myR[6];
	if (lastWindow != myWindow || (displayStatus & 0x10)) { //Only clear when changing from one window to another, or when force is enabled
		lcd_clear();
		
		
		lastWindow = window;
		window = myWindow;
		displayStatus &= ~(0x10);
	}
	lcd_home();
		
	char out[16] ="";
	uint8_t pt;
	uint8_t myByte;
	uint8_t ret = 0;
	char myData[13];
	switch (window) {
		/*case k1stSetup:
			lcd_string("SETUP");
			break;*/
		case k1stDo1cal:
			lcd_string("1. Kalibration");
			set_cursor(0,2);
			lcd_string("1000 Ohm");
			break;
		case k1stDo2cal:
			lcd_string("2. Kalibration");
			set_cursor(0,2);
			lcd_string("1400 Ohm");
			break;
		case k1stShow1cal:
			lcd_string("zeige 1. Kal.");
			break;
		case k1stShow2cal:
			lcd_string("zeige 2. Kal.");
			break;
		case k1stShowClock:
			lcd_string("zeige Datum/Uhr");
			break;
		case k1stDoClock:
			lcd_string("Datum/Uhr einst.");
			break;
		case k1stSetInterval:
			lcd_string("SD-Interval");
			break;
		case k2ndSetInterval:
			getIntervalString(out, SDInterval);
			lcd_string(out);
			break;
		case k1stReset:
			lcd_string("Reset to factory");
			set_cursor(0,2);
			lcd_string("settings...");
			break;
		case k2ndReset:
			lcd_string("Reset...  sure?");
			break;
		case k2ndDo1cal: //calib1000
			lcd_string("1.0k ");
			for (pt=0; pt <8; ++pt) {
				if (pt == 4) set_cursor(5,2);
				/*if (c1000[pt] != 1000) {
					lcd_string("OK ");
				} else {
					lcd_string("?? ");
				}*/
				if (isCal1000 & (1 << pt)) lcd_string("OK "); else lcd_string("?? ");
			}
			set_cursor(0,2);
			
			if (showR == 9999) 
				sprintf(myR,"%s","inf.");
			else 
				sprintf(myR,"%d",showR);
			lcd_string(myR);
			
			break;
		case k2ndShow1cal: // show calib1000
			for (pt=0; pt <8; ++pt) {
				if (pt == 4) set_cursor(0,2);
				char out[4];
				sprintf(out,"% 4d",c1000[pt]);
				lcd_string(out);
			}
			
			break;
		case k2ndDo2cal: //calib1400
			lcd_string("1.4k ");
			for (pt=0; pt <8; ++pt) {
				if (pt == 4) set_cursor(5,2);
				if (isCal1400 & (1 << pt)) lcd_string("OK "); else lcd_string("?? ");
				
			}
			set_cursor(0,2);
			if (showR == 9999) 
				sprintf(myR,"%s","inf.");
			else 
				sprintf(myR,"%d",showR);
			lcd_string(myR);
			
			break;
		case k2ndShow2cal: // show calib1400
			for (pt=0; pt <8; ++pt) {
				if (pt == 4) set_cursor(0,2);
				char out[4];
				sprintf(out,"% 4d",c1400[pt]);
				lcd_string(out);
			}
			
			break;
		case k1stNormal: //normal operation
			set_cursor(0,1);
			
			
			for (pt=0;pt<8;pt++) {
				if (pt ==4) set_cursor(0,2);
				int16_t myTemp = 0;
				uint8_t tempPos = 0;
				for (tempPos = 0; tempPos < 10; tempPos++) {
					myTemp += temps[tempPos][pt];
				}
				myTemp /= 10;
				
				if ((displayStatus >> 3) & 0x01) { // all necessarry temp-readouts done
					 if (myTemp > 120) {
						sprintf(out," NC ");
						lcd_string(out);
					} else if (myTemp < 100 && myTemp >= 0){
						if (myTemp <10) lcd_data(' ');
						lcd_data(' ');
						sprintf(out,"%d",myTemp);
						lcd_string(out);
						lcd_data(0x00);
					} else {
						if (myTemp >-10 && myTemp < 0) lcd_data(' ');
						sprintf(out,"%d",myTemp);
						lcd_string(out);
						lcd_data(0x00);
					}
				} else { // not enough temp-readouts
					sprintf(out," ?? ");
					lcd_string(out);
				}
				
			}
			break;
			
		case k2ndDoClock:
		case k2ndDoClockYear:
		case k2ndDoClockMonth:
		case k2ndDoClockDay:
		case k2ndDoClockHour:
		case k2ndDoClockMinute:
		case k2ndDoClockSec:
		case k2ndShowClock:
			set_cursor(0,1);
			
			//I2CgetDateStr(myData);
			char space = ' ';
			if (window == k2ndShowClock) 
				sprintf(out,"%02d-%02d-%02d",datime[0],datime[1],datime[2]);
			else 
				sprintf(out,"%02d-%02d-%02d",newDatime[0],newDatime[1],newDatime[2]);
			if ((window == k2ndDoClockYear) && (myISRs & 0x04)) {out[0] = space; out[1] = space; }
			if ((window == k2ndDoClockMonth) && (myISRs & 0x04)) {out[3] = space; out[4] = space; }
			if ((window == k2ndDoClockDay) && (myISRs & 0x04)) {out[6] = space; out[7] = space; }
			lcd_string(out);
			
			set_cursor(0,2);
			//I2CgetTimeStr(myData);
			if (window == k2ndShowClock) 
				sprintf(out,"%02d:%02d:%02d",datime[3], datime[4], datime[5]);
			else 
				sprintf(out,"%02d:%02d:%02d",newDatime[3], newDatime[4], newDatime[5]);
			if ((window == k2ndDoClockHour) && (myISRs & 0x04)) {out[0] = space; out[1] = space; }
			if ((window == k2ndDoClockMinute) && (myISRs & 0x04)) {out[3] = space; out[4] = space; }
			if ((window == k2ndDoClockSec) && (myISRs & 0x04)) {out[6] = space; out[7] = space; }
			lcd_string(out);
			
			break;
		case kSaved:
			lcd_string("Saved to EEPROM!");
			break;
	}
	if (displayStatus & 0x04) { //SD operation done
		set_cursor(14,2);
		lcd_string("SD");
		displayStatus &= ~(0x04);
		if (window != k1stNormal) displayStatus |= 0x10; //force clear LCD
	}	
	displayStatus |= (1 << 0); //set bit 0, Means windows up to date
	
}

/* Handles all actions initiated through the HMI */
void menuLoop() {
	uint8_t pt;
	//char msg[80];
	if (debounce(&PIND, SW_NEXT) ) {
		if (window < 0x10) { // first layer
			window++;
			if (window > 0x08 && window < 0x10) window = k1stNormal;
		} else if (window >0x30 && window < 0x38) { //Setting the clock
			switch (window) {
				case k2ndDoClockYear:
					displayStatus &= ~(1 << 0);
					newDatime[0]++;
					if (newDatime[0] > 99) newDatime[0] = 0;
					break;
				case k2ndDoClockMonth:
					displayStatus &= ~(1 << 0);
					newDatime[1]++;
					if (newDatime[1] > 12) newDatime[1] = 0;
					break;
				case k2ndDoClockDay:
					displayStatus &= ~(1 << 0);
					newDatime[2]++;
					if (newDatime[2] > 31) newDatime[2] = 0;
					break;
				case k2ndDoClockHour:
					displayStatus &= ~(1 << 0);
					newDatime[3]++;
					if (newDatime[3] > 24) newDatime[3] = 0;
					break;
				case k2ndDoClockMinute:
					displayStatus &= ~(1 << 0);
					newDatime[4]++;
					if (newDatime[4] > 59) newDatime[4] = 0;
					break;
				case k2ndDoClockSec:
					displayStatus &= ~(1 << 0);
					newDatime[5]++;
					if (newDatime[5] > 59) newDatime[5] = 0;
					break;
			}
		} else if (window == k2ndSetInterval) {
			SDInterval++;
			if (SDInterval > k1d) SDInterval = k1Sec;
			displayStatus |= 0x10; //Force update
		} else { //acts as cancel
			window = k1stNormal;
		}
	}
	
	if (debounce(&PIND, SW_SET)) {
		switch (window) {
			/* Simple goto from first layer to second layer */
			case k1stDo1cal: //goto do first calib
				displayStatus &= ~(1 << 0);
				window = k2ndDo1cal;
				break;
			case k1stDo2cal: //goto do second calib
				displayStatus &= ~(1 << 0);
				window = k2ndDo2cal;
				break;
			case k1stShow1cal: // goto show first calib
				displayStatus &= ~(1 << 0);
				window = k2ndShow1cal;
				break;
			case k1stShow2cal: //goto show second calib
				displayStatus &= ~(1 << 0);
				window = k2ndShow2cal;
				break;
			case k1stShowClock: //goto show clock
				displayStatus &= ~(1 << 0);
				window = k2ndShowClock;
				break;
			case k1stDoClock: //goto change clock
				displayStatus &= ~(1 << 0);
				uint8_t j;
				for (j=0; j < 6; j++) {
					newDatime[j] = datime[j];
				}
				window = k2ndDoClockYear;
				break;
			case k1stSetInterval: //goto change interval
				displayStatus &= ~(1 << 0);
				window = k2ndSetInterval;
				break;
			case k1stReset: //goto reset
				displayStatus &= ~(1 << 0);
				window = k2ndReset;
				break;
				
				
			/* Press Set inside calibrations */
			case k2ndDo1cal: // save cal1000 to eeprom
				displayStatus &= ~(1 << 0);
				eeprom_update_byte(&eeIsCal1000, isCal1000);
				for (pt=0;pt<8;++pt) {
					eeprom_update_word(&eeC1000[pt], c1000[pt]);
				}
				
				//sprintf(msg,logTextCal1000, c1000[0], c1000[1], c1000[2], c1000[3], c1000[4], c1000[5], c1000[6], c1000[7]);
				//LogMessage(msg);
				//window = k1stDo1cal;
				window = kSaved;
				break;
			case k2ndDo2cal: // save cal1400 to eeprom
				displayStatus &= ~(1 << 0);
				eeprom_update_byte(&eeIsCal1400, isCal1400);
				for (pt=0;pt<8;++pt) {
					eeprom_update_word(&eeC1400[pt], c1400[pt]);
				}
				
				//sprintf(msg,"Kalibration für 1.4kOhm verändert: %d, %d, %d, %d, %d, %d, %d, %d",c1400[0], c1400[1], c1400[2], c1400[3], c1400[4], c1400[5], c1400[6], c1400[7]);
				//LogMessage(msg);
				//window = k1stDo2cal;
				window = kSaved;
				break;
			
			/* Press Set inside Show calib */
			case k2ndShow1cal:
				displayStatus &= ~(1 << 0);
				window = k1stShow1cal;
				break;
			case k2ndShow2cal:
				displayStatus &= ~(1 << 0);
				window = k1stShow2cal;
				break;
				
				
			/* Press Set inside 'set the clock' */
			case k2ndDoClockYear: //no break!!!
			case k2ndDoClockMonth: //no break!!!
			case k2ndDoClockDay: //no break!!!
			case k2ndDoClockHour: //no break!!!
			case k2ndDoClockMinute: 
				window++;
				break;
			
			case k2ndDoClockSec:
			
				//sprintf(msg,"Datum/Uhrzeit geändert auf: %02d-%02d-%02d %02d:%02d:%02d", newDatime[0], newDatime[1], newDatime[2],newDatime[3], newDatime[4], newDatime[5]);
				//LogMessage(msg);
				
				//save!!!!
				I2CsetDate(newDatime[0], newDatime[1], newDatime[2]);
				I2CsetTime(newDatime[3], newDatime[4], newDatime[5]);
				
				for (j=0; j < 6; j++) {
					datime[j] = newDatime[j];
				}
				
				window = kSaved;
				break;
			
			/* Press Set inside SetInterval */
			case k2ndSetInterval:
				displayStatus &= ~(1 << 0);
				//save
				eeprom_update_byte(&eeSDInterval, SDInterval);
				char msg2[12];
				getIntervalString(msg2,SDInterval);
				//sprintf(msg,"SD-Intervall auf %s geändert", msg2);
				
				//LogMessage(msg);
				window = kSaved;
				
				break;
				
			case k2ndReset:
				displayStatus &= ~(1 << 0);
				//Default values are
				for (pt=0; pt < 8; pt++) {
					c1000[pt] = 1000; 
					c1400[pt] = 1400;
				}
				isCal1000 = 0;
				isCal1400 = 0;
				SDInterval = k1Min;
				
				//Saving them into EEPROM
				eeprom_update_byte(&eeIsCal1400, isCal1400);
				eeprom_update_byte(&eeIsCal1000, isCal1000);
				for (pt=0;pt<8;++pt) {
					eeprom_update_word(&eeC1400[pt], c1400[pt]);
					eeprom_update_word(&eeC1000[pt], c1000[pt]);
				}
				eeprom_update_byte(&eeSDInterval, SDInterval);
				//sprintf(msg,"Reset auf Factory-Settings");
				//LogMessage(msg);
				
				window = kSaved;
				break;
				
			default:
				displayStatus &= ~(1 << 0);
				window = k1stNormal;
		}
	}
}
