// $Id$

// Copyright (C) 2009 Dr. Georg Troska <georg.troska@tu-dortmund.de>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include "pt1000.h"
#include "spi.h"


void enablePt1000(uint8_t channel) {
	switch (channel) {
		case 0:
			SPI_sendToPT(0x80);
			break;
		case 1:
			SPI_sendToPT(0x40);
			break;
		case 2:
			SPI_sendToPT(0x20);
			break;
		case 3:
			SPI_sendToPT(0x10);
			break;
		case 4:
			SPI_sendToPT(0x08);
			break;
		case 5:
			SPI_sendToPT(0x04);
			break;
		case 6:
			SPI_sendToPT(0x02);
			break;
		case 7:
			SPI_sendToPT(0x01);
			break;
		default:
			SPI_sendToPT(0x00);
	}
}
void disablePt1000() {
	enablePt1000(255);
}

uint16_t getR(uint8_t channel) {
	
	//in mV
	const uint16_t Uv = 3300;
	//const float V = 150./10.9; //Widerstandsverhältnis
	const float V = 150./10; //Widerstandsverhältnis
	
	const float Rfix = 680./(10000+680);//Widerstandsverhältnis in der Wheatstoneschen Brücke: R4/(R3+R4)
	//const uint16_t RnextPt = 10080;//10180; //Widerstand neben Pt1000
	const uint16_t RnextPt = 10000;//10180; //Widerstand neben Pt1000
	/* Formulars:
	 * adc = Ua/Uv*1024
	 * Ua = V*(Ue2-Ue1)
	 * Udiff = Ue2-Ue1
	 * Wheatstonesche Brücke:
	 * Ue1 (fester Spannungsteiler) = 10k, 680 an 5V -> 680/10680*5V = 0.318V
	 * Ue2 (var. Spannungsteiler) = 10k, Pt1000 an 5V -> pt/(10k+pt) = ???
	 * 
	 * */
	float Uadc, Ue2;
	uint16_t adc;
	
	switch (channel) {
		case 0:
			adc = ADC_readAvg(5,3);
			break;
		case 1:
			adc = ADC_readAvg(4,3);
			break;
		case 2:
			adc = ADC_readAvg(3,3);
			break;
		case 3:
			adc = ADC_readAvg(2,3);
			break;
		case 4:
			adc = ADC_readAvg(1,3);
			break;
		case 5:
			adc = ADC_readAvg(0,3);
			break;
		case 6:
			adc = ADC_readAvg(7,3);
			break;
		case 7:
			adc = ADC_readAvg(6,3);
			break;
		default:
			return 0xffff;
	}
	
	Uadc = adc/1024.*Uv;
	Ue2 = Uadc/V + Rfix*Uv;
	return Ue2*RnextPt/(Uv-Ue2);
	
}

int16_t getT(uint16_t R) {
	const float a=3.9083E-3;
	const float b=-5.775E-7;
	const float R0=1000.00;

	return (-a/(2*b)-sqrt(R/(R0*b)-1/b+(a/(2*b))*(a/(2*b))));

}
