// $Id$

// Copyright (C) 2009 Dr. Georg Troska <georg.troska@tu-dortmund.de>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#include "sd.h"
#include <util/delay.h>
#include <avr/interrupt.h>



uint8_t find_file_in_dir(struct fat_fs_struct* fs, struct fat_dir_struct* dd, const char* name, struct fat_dir_entry_struct* dir_entry)
{
    while(fat_read_dir(dd, dir_entry))
    {
        if(strcmp(dir_entry->long_name, name) == 0)
        {
            fat_reset_dir(dd);
            return 1;
        }
    }

    return 0;
}

struct fat_file_struct* open_file_in_dir(struct fat_fs_struct* fs, struct fat_dir_struct* dd, const char* name)
{
    struct fat_dir_entry_struct file_entry;
    if(!find_file_in_dir(fs, dd, name, &file_entry))
        return 0;

    return fat_open_file(fs, &file_entry);
}


uint16_t SDappendFile(struct fat_fs_struct *fs, char * path, char * filename, char * writeStr) {
	uint8_t ret = 1; if (!fs || !path || !filename || !writeStr) ret = 0;
	//Receiving the directory entry
	struct fat_dir_entry_struct directory;
	if (!fat_get_dir_entry_of_path(fs, path, &directory) && ret) ret = 0;
	
	//Receiving the directory
	struct fat_dir_struct* dd;
	dd = fat_open_dir(fs, &directory);
	if (!dd && ret) ret = 0;
	
	//Try to open the given filename
	struct fat_file_struct* filehandle;
	filehandle = open_file_in_dir(fs, dd, filename);
	if (!filehandle) { //No, the file does not exisit
		fat_create_file(dd, filename, &directory); //Create it
		filehandle = fat_open_file(fs, &directory); //And open the new fresh file instead
	}
	//Still not possible to access that file in that directory
	if (!filehandle && ret) ret = 0;
	
	//Move the filehandle to the end of the file, so that the further strings can be appended	
	int32_t offset = 0;
	if (!fat_seek_file(filehandle, &offset, FAT_SEEK_END) && ret) ret = 0;

	//Writing to the file
	intptr_t bufferLength;
	bufferLength = fat_write_file(filehandle, writeStr, strlen(writeStr));
	if (bufferLength <= 0 && ret) ret = 0;	
	
	fat_close_file(filehandle);	
	
	//Syncing
	if (!sd_raw_sync() && ret) ret = 0;
	
	fat_close_dir(dd); ///???
	/*
	if (!ret) { //Seems the SD-Card was removed
		cli();
		SDUnload();
		lcd_clear();
		lcd_home();
		lcd_string("SD entfernt");
		_delay_ms(500);
		sei();
		return 0;
	}*/
	if (!ret) return 0;
	
	return bufferLength;

}
/* NOT WORKING????
uint8_t SDopenFilesystemRW(struct partition_struct *part, struct fat_fs_struct *fs) {
	part = partition_open(sd_raw_read, sd_raw_read_interval, sd_raw_write, sd_raw_write_interval, 0);
	//if (!part) return 0;
	fs = fat_open(part);
	if (fs) return 1;
	//return 0;
}
*/


void SDTryLoading() {
	sd_raw_init();
	uint8_t ret = 0;
	partition = partition_open(sd_raw_read, sd_raw_read_interval, sd_raw_write, sd_raw_write_interval, 0);
	fs = fat_open(partition);
	//if (fs) ret = SDappendFile(fs, "/", "DATA.TXT", "Neustart\n");
}

void SDUnload() {
	/* close file system */
	fat_close(fs);

	/* close partition */
	partition_close(partition);
}

void LogMessage (char msg[]) {
	char mychar[32];
	char SQLDatime[22];
	getSQLDatime(SQLDatime);
	sprintf(mychar, "%s\t%s\n",SQLDatime, msg);
	if (fs) SDappendFile(fs, "/", "LOG.TXT", mychar);
}
